import { fieldsToJSONSchema } from '../../utilities/configToJSONSchema.js';
export function generateJobsJSONSchemas(config, jobsConfig, interfaceNameDefinitions, /**
   * Used for relationship fields, to determine whether to use a string or number type for the ID.
   * While there is a default ID field type set by the db adapter, they can differ on a collection-level
   * if they have custom ID fields.
   */ collectionIDFieldTypes) {
    const properties = {
        tasks: {},
        workflows: {}
    };
    const definitions = new Map();
    if (jobsConfig?.tasks?.length) {
        for (const task of jobsConfig.tasks){
            if (task?.inputSchema?.length) {
                const inputJsonSchema = fieldsToJSONSchema(collectionIDFieldTypes, task.inputSchema, interfaceNameDefinitions, config);
                const fullInputJsonSchema = {
                    type: 'object',
                    additionalProperties: false,
                    properties: inputJsonSchema.properties,
                    required: inputJsonSchema.required
                };
                definitions.set(`Task${task.slug}Input`, fullInputJsonSchema);
            }
            if (task?.outputSchema?.length) {
                const outputJsonSchema = fieldsToJSONSchema(collectionIDFieldTypes, task.outputSchema, interfaceNameDefinitions, config);
                const fullOutputJsonSchema = {
                    type: 'object',
                    additionalProperties: false,
                    properties: outputJsonSchema.properties,
                    required: outputJsonSchema.required
                };
                definitions.set(`Task${task.slug}Output`, fullOutputJsonSchema);
            }
        }
        // Now add properties.tasks definition that references the types in definitions keyed by task slug:
        properties.tasks = {
            type: 'object',
            additionalProperties: false,
            properties: Object.fromEntries(jobsConfig.tasks.map((task)=>{
                const toReturn = {
                    type: 'object',
                    additionalProperties: false,
                    properties: {
                        input: {}
                    },
                    required: [
                        'input',
                        'output'
                    ]
                };
                if (task.inputSchema?.length) {
                    toReturn.required.push('input');
                    toReturn.properties.input = {
                        $ref: `#/definitions/Task${task.slug}Input`
                    };
                }
                if (task.outputSchema?.length) {
                    toReturn.required.push('output');
                    toReturn.properties.output = {
                        $ref: `#/definitions/Task${task.slug}Output`
                    };
                }
                return [
                    task.slug,
                    toReturn
                ];
            })),
            required: jobsConfig.tasks.map((task)=>task.slug)
        };
    }
    if (jobsConfig?.workflows?.length) {
        for (const workflow of jobsConfig.workflows){
            if (workflow?.inputSchema?.length) {
                const inputJsonSchema = fieldsToJSONSchema(collectionIDFieldTypes, workflow.inputSchema, interfaceNameDefinitions, config);
                const fullInputJsonSchema = {
                    type: 'object',
                    additionalProperties: false,
                    properties: inputJsonSchema.properties,
                    required: inputJsonSchema.required
                };
                definitions.set(`Workflow${workflow.slug}Input`, fullInputJsonSchema);
            }
            properties.workflows = {
                type: 'object',
                additionalProperties: false,
                properties: Object.fromEntries(jobsConfig.workflows.map((workflow)=>{
                    const toReturn = {
                        type: 'object',
                        additionalProperties: false,
                        properties: {
                            input: {}
                        },
                        required: [
                            'input'
                        ]
                    };
                    if (workflow.inputSchema?.length) {
                        toReturn.required.push('input');
                        toReturn.properties.input = {
                            $ref: `#/definitions/Workflow${workflow.slug}Input`
                        };
                    }
                    return [
                        workflow.slug,
                        toReturn
                    ];
                })),
                required: jobsConfig.tasks.map((task)=>task.slug)
            };
        }
    }
    return {
        definitions,
        properties
    };
}

//# sourceMappingURL=generateJobsJSONSchemas.js.map