import { Forbidden } from '../../../errors/Forbidden.js';
import { commitTransaction } from '../../../utilities/commitTransaction.js';
import { initTransaction } from '../../../utilities/initTransaction.js';
import isolateObjectProperty from '../../../utilities/isolateObjectProperty.js';
import { getJobStatus } from '../../utilities/getJobStatus.js';
import { runJob } from '../runJob/index.js';
export const runAllJobs = async ({ limit = 10, overrideAccess, queue, req })=>{
    if (!overrideAccess) {
        const hasAccess = await req.payload.config.jobs.access.run({
            req
        });
        if (!hasAccess) {
            throw new Forbidden(req.t);
        }
    }
    const where = {
        and: [
            {
                completedAt: {
                    exists: false
                }
            },
            {
                hasError: {
                    not_equals: true
                }
            },
            {
                processing: {
                    equals: false
                }
            },
            {
                or: [
                    {
                        waitUntil: {
                            exists: false
                        }
                    },
                    {
                        waitUntil: {
                            less_than: new Date()
                        }
                    }
                ]
            }
        ]
    };
    if (queue) {
        where.and.push({
            queue: {
                equals: queue
            }
        });
    }
    req.payload.logger.info({
        msg: `Querying for ${limit} jobs.`
    });
    const jobsQuery = await req.payload.find({
        collection: 'payload-jobs',
        depth: req.payload.config.jobs.depth,
        limit,
        showHiddenFields: true,
        where
    });
    const jobs = jobsQuery.docs.reduce((acc, job)=>{
        if (job.seenByWorker) {
            acc.existingJobs.push(job);
        } else {
            acc.newJobs.push(job);
        }
        return acc;
    }, {
        existingJobs: [],
        newJobs: []
    });
    const numJobs = jobsQuery.totalDocs;
    if (!numJobs) {
        return {
            noJobsRemaining: true
        };
    }
    if (jobs.newJobs.length) {
        req.payload.logger.info(`${numJobs} job(s) found, ${jobs.newJobs.length} of which are new. Running ${limit} job(s).`);
    }
    const jobPromises = jobsQuery.docs.map(async (job)=>{
        if (!job.workflowSlug && !job.taskSlug) {
            throw new Error('Job must have either a workflowSlug or a taskSlug');
        }
        const workflowConfig = job.workflowSlug ? req.payload.config.jobs.workflows.find(({ slug })=>slug === job.workflowSlug) : {
            slug: 'singleTask',
            handler: async ({ job, runTask })=>{
                await runTask({
                    id: '1',
                    input: job.input,
                    task: job.taskSlug
                });
            }
        };
        if (!workflowConfig) {
            return null // Skip jobs with no workflow configuration
            ;
        }
        const newReq = isolateObjectProperty(req, 'transactionID');
        // Create a transaction so that all seeding happens in one transaction
        await initTransaction(newReq);
        const result = await runJob({
            job,
            jobTasksStatus: getJobStatus({
                job,
                tasksConfig: req.payload.config.jobs.tasks
            }),
            // Each job should have its own transaction. Can't have multiple running jobs in parallel on same transaction
            req: newReq,
            workflowConfig
        });
        // Finalise transactiojn
        await commitTransaction(newReq);
        return {
            id: job.id,
            result
        };
    });
    const resultsArray = await Promise.all(jobPromises);
    const resultsObject = resultsArray.reduce((acc, cur)=>{
        if (cur !== null) {
            // Check if there's a valid result to include
            acc[cur.id] = cur.result;
        }
        return acc;
    }, {});
    let noJobsRemaining = true;
    for(const jobID in resultsObject){
        const jobResult = resultsObject[jobID];
        if (jobResult.status === 'error') {
            noJobsRemaining = false // Can be retried
            ;
        }
    }
    return {
        jobStatus: resultsObject,
        noJobsRemaining
    };
};

//# sourceMappingURL=index.js.map